#[non_exhaustive]pub enum AppendSessionError {
Append(AppendError),
AckTimeout,
ServerDisconnected,
StreamClosedEarly,
SessionClosed,
SessionClosing,
SessionDropped,
InvalidAck(String),
}Expand description
Errors returned by an append session.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Append(AppendError)
An error with the append request underlying the session.
AckTimeout
An append acknowledgement timed out.
ServerDisconnected
The server disconnected during the session.
StreamClosedEarly
The response stream closed while appends were in flight.
SessionClosed
The session was already closed.
SessionClosing
The session is closing.
SessionDropped
The session was dropped without being closed.
InvalidAck(String)
The server returned an invalid append acknowledgement.
Implementations
Sourceimpl AppendSessionError
impl AppendSessionError
Sourcepub fn is_retryable(&self) -> bool
pub fn is_retryable(&self) -> bool
Whether retrying the operation is safe or sensible.
Sourcepub fn has_no_side_effects(&self) -> bool
pub fn has_no_side_effects(&self) -> bool
Whether retrying the operation cannot duplicate a mutation.
Sourcepub fn request_error(&self) -> Option<&RequestError>
pub fn request_error(&self) -> Option<&RequestError>
Return the underlying request error, if present.