ErrorCode

Enum ErrorCode 

#[non_exhaustive]
pub enum ErrorCode {
Show 28 variants AccessTokenNotFound, Authn, BadFrame, BadHeader, BadJson, BadPath, BadProto, BadQuery, BasinDeletionPending, BasinNotFound, ClientHangup, DecryptionFailed, HotServer, Invalid, NotImplemented, Other, PermissionDenied, QuotaExhausted, RateLimited, RequestTimeout, ResourceAlreadyExists, ServerDraining, Storage, StreamDeletionPending, StreamNotFound, TransactionConflict, Unavailable, UpstreamTimeout,
}

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


AccessTokenNotFound



Authn



BadFrame



BadHeader



BadJson



BadPath



BadProto



BadQuery



BasinDeletionPending



BasinNotFound



ClientHangup



DecryptionFailed



HotServer



Invalid



NotImplemented



Other



PermissionDenied



QuotaExhausted



RateLimited



RequestTimeout



ResourceAlreadyExists



ServerDraining



Storage



StreamDeletionPending



StreamNotFound



TransactionConflict



Unavailable



UpstreamTimeout

Implementations

Source

impl ErrorCode

pub fn is_auth_error(self) -> bool

Whether this code represents an authentication or authorization failure.

pub fn status(self) -> StatusCode

HTTP status associated with this error code.

pub fn is_retryable(self) -> bool

Whether retrying an operation that returned this code is sensible.

pub fn has_no_side_effects(self) -> bool

Whether the server guarantees that an operation returning this code had no side effects.