#[non_exhaustive]pub enum ReadSessionRetryPolicy {
Budgeted,
Indefinite,
}Expand description
Retry policy for a continuous read session.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Budgeted
Stop after the retry budget configured by RetryConfig is exhausted.
Indefinite
Keep retrying retryable failures after the configured retry budget is exhausted.
This also applies while establishing the initial session, so
read_session may remain pending through retryable
failures until it connects or the future is cancelled.