BasinConfig

Struct BasinConfig 

#[non_exhaustive]
pub struct BasinConfig { pub default_stream_config: Option<StreamConfig>, pub stream_cipher: Option<EncryptionAlgorithm>, pub create_stream_on_append: bool, pub create_stream_on_read: bool, }
Configuration for a basin.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
default_stream_config: Option<StreamConfig>

Default configuration for all streams in the basin.

See StreamConfig for defaults.

stream_cipher: Option<EncryptionAlgorithm>

Encryption algorithm to apply to newly created streams in the basin.

create_stream_on_append: bool

Whether to create stream on append if it doesn’t exist using default stream configuration.

Defaults to false.

create_stream_on_read: bool

Whether to create stream on read if it doesn’t exist using default stream configuration.

Defaults to false.

Implementations

Source

impl BasinConfig

pub fn new() -> Self

Create a new BasinConfig with default settings.

Examples found in repository?
examples/create_basin.rs (line 18)
7async fn main() -> Result<(), Box<dyn std::error::Error>> {
8    let access_token =
9        std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
10    let basin_name: BasinName = std::env::var("S2_BASIN")
11        .map_err(|_| "S2_BASIN env var not set")?
12        .parse()?;
13
14    let config = S2Config::new(access_token);
15    let s2 = S2::new(config)?;
16
17    let input = CreateBasinInput::new(basin_name.clone()).with_config(
18        BasinConfig::new().with_default_stream_config(
19            StreamConfig::new().with_retention_policy(RetentionPolicy::Age(10 * 24 * 60 * 60)),
20        ),
21    );
22    let basin_info = s2.create_basin(input).await?;
23    println!("{basin_info:#?}");
24
25    let basin_config = s2.get_basin_config(basin_name).await?;
26    println!("{basin_config:#?}");
27
28    Ok(())
29}
pub fn with_default_stream_config(self, config: StreamConfig) -> Self

Set the default configuration for all streams in the basin.

Examples found in repository?
examples/create_basin.rs (lines 18-20)
7async fn main() -> Result<(), Box<dyn std::error::Error>> {
8    let access_token =
9        std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
10    let basin_name: BasinName = std::env::var("S2_BASIN")
11        .map_err(|_| "S2_BASIN env var not set")?
12        .parse()?;
13
14    let config = S2Config::new(access_token);
15    let s2 = S2::new(config)?;
16
17    let input = CreateBasinInput::new(basin_name.clone()).with_config(
18        BasinConfig::new().with_default_stream_config(
19            StreamConfig::new().with_retention_policy(RetentionPolicy::Age(10 * 24 * 60 * 60)),
20        ),
21    );
22    let basin_info = s2.create_basin(input).await?;
23    println!("{basin_info:#?}");
24
25    let basin_config = s2.get_basin_config(basin_name).await?;
26    println!("{basin_config:#?}");
27
28    Ok(())
29}
pub fn with_stream_cipher(self, stream_cipher: EncryptionAlgorithm) -> Self

Set the encryption algorithm to apply to newly created streams in the basin.

pub fn with_create_stream_on_append(self, create_stream_on_append: bool) -> Self

Set whether to create stream on append if it doesn’t exist using default stream configuration.

pub fn with_create_stream_on_read(self, create_stream_on_read: bool) -> Self

Set whether to create stream on read if it doesn’t exist using default stream configuration.