ReadLimits

Struct ReadLimits 

#[non_exhaustive]
pub struct ReadLimits { pub count: Option<usize>, pub bytes: Option<usize>, }
Limits on how much to read.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
count: Option<usize>

Limit on number of records.

Defaults to 1000 for non-streaming read.

bytes: Option<usize>

Limit on total metered bytes of records.

Defaults to 1MiB for non-streaming read.

Implementations

Source

impl ReadLimits

pub fn new() -> Self

Create a new ReadLimits with default values.

Examples found in repository?
examples/get_latest_record.rs (line 24)
9async fn main() -> Result<(), Box<dyn std::error::Error>> {
10    let access_token =
11        std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
12    let basin_name: BasinName = std::env::var("S2_BASIN")
13        .map_err(|_| "S2_BASIN env var not set")?
14        .parse()?;
15    let stream_name: StreamName = std::env::var("S2_STREAM")
16        .map_err(|_| "S2_STREAM env var not set")?
17        .parse()?;
18
19    let s2 = S2::new(S2Config::new(access_token))?;
20    let stream = s2.basin(basin_name).stream(stream_name);
21
22    let input = ReadInput::new()
23        .with_start(ReadStart::new().with_from(ReadFrom::TailOffset(1)))
24        .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(1)));
25    let batch = stream.read(input).await?;
26    println!("{batch:#?}");
27
28    Ok(())
29}
More examples
Hide additional examples
examples/docs_encryption.rs (line 64)
15async fn main() -> Result<(), Box<dyn std::error::Error>> {
16    let access_token = std::env::var("S2_ACCESS_TOKEN")?;
17    let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
18    let stream_name: StreamName = format!(
19        "docs-encryption-{}",
20        std::time::SystemTime::now()
21            .duration_since(std::time::UNIX_EPOCH)?
22            .as_millis()
23    )
24    .parse()?;
25
26    let client = S2::new(S2Config::new(access_token))?;
27
28    // ANCHOR: basin-cipher
29    client
30        .create_basin(
31            CreateBasinInput::new(basin_name.clone())
32                .with_config(BasinConfig::new().with_stream_cipher(EncryptionAlgorithm::Aegis256)),
33        )
34        .await?;
35
36    client
37        .reconfigure_basin(ReconfigureBasinInput::new(
38            basin_name.clone(),
39            BasinReconfiguration::new().with_stream_cipher(EncryptionAlgorithm::Aes256Gcm),
40        ))
41        .await?;
42    // ANCHOR_END: basin-cipher
43
44    let basin = client.basin(basin_name.clone());
45    basin
46        .create_stream(CreateStreamInput::new(stream_name.clone()))
47        .await?;
48
49    // ANCHOR: append-read
50    let stream = basin
51        .stream(stream_name.clone())
52        .with_encryption_key(std::env::var("S2_ENCRYPTION_KEY")?.parse()?);
53
54    stream
55        .append(AppendInput::new(AppendRecordBatch::try_from_iter([
56            AppendRecord::new("top secret")?,
57        ])?))
58        .await?;
59
60    let batch = stream
61        .read(
62            ReadInput::new()
63                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
64                .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(10))),
65        )
66        .await?;
67    // ANCHOR_END: append-read
68
69    println!("Read {} encrypted record(s)", batch.records.len());
70
71    basin
72        .delete_stream(DeleteStreamInput::new(stream_name))
73        .await?;
74
75    Ok(())
76}
examples/docs_streams.rs (line 62)
20async fn main() -> Result<(), Box<dyn std::error::Error>> {
21    let access_token = std::env::var("S2_ACCESS_TOKEN")?;
22    let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
23
24    let client = S2::new(S2Config::new(access_token))?;
25    let basin = client.basin(basin_name);
26
27    // Create a temporary stream for examples
28    let stream_name: StreamName = format!(
29        "docs-streams-{}",
30        std::time::SystemTime::now()
31            .duration_since(std::time::UNIX_EPOCH)?
32            .as_millis()
33    )
34    .parse()?;
35    basin
36        .create_stream(s2_sdk::types::CreateStreamInput::new(stream_name.clone()))
37        .await?;
38
39    // ANCHOR: simple-append
40    let stream = basin.stream(stream_name.clone());
41
42    let ack = stream
43        .append(AppendInput::new(AppendRecordBatch::try_from_iter([
44            AppendRecord::new("first event")?,
45            AppendRecord::new("second event")?,
46        ])?))
47        .await?;
48
49    // ack tells us where the records landed
50    println!(
51        "Wrote records {} through {}",
52        ack.start.seq_num,
53        ack.end.seq_num - 1
54    );
55    // ANCHOR_END: simple-append
56
57    // ANCHOR: simple-read
58    let batch = stream
59        .read(
60            ReadInput::new()
61                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
62                .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(100))),
63        )
64        .await?;
65
66    for record in batch.records {
67        println!("[{}] {:?}", record.seq_num, record.body);
68    }
69    // ANCHOR_END: simple-read
70
71    // ANCHOR: append-session
72    let session = stream.append_session(AppendSessionConfig::new());
73
74    // Submit a batch - this enqueues it and returns a ticket
75    let records = AppendRecordBatch::try_from_iter([
76        AppendRecord::new("event-1")?,
77        AppendRecord::new("event-2")?,
78    ])?;
79    let ticket = session.submit(AppendInput::new(records)).await?;
80
81    // Wait for durability
82    let ack = ticket.await?;
83    println!("Durable at seqNum {}", ack.start.seq_num);
84
85    session.close().await?;
86    // ANCHOR_END: append-session
87
88    // ANCHOR: producer
89    let producer = stream.producer(
90        ProducerConfig::new()
91            .with_batching(BatchingConfig::new().with_linger(Duration::from_millis(5))),
92    );
93
94    // Submit individual records
95    let ticket = producer.submit(AppendRecord::new("my event")?).await?;
96
97    // Get the exact sequence number
98    let ack = ticket.await?;
99    println!("Record durable at seqNum {}", ack.seq_num);
100
101    producer.close().await?;
102    // ANCHOR_END: producer
103
104    // ANCHOR: check-tail
105    let tail = stream.check_tail().await?;
106    println!("Stream has {} records", tail.seq_num);
107    // ANCHOR_END: check-tail
108
109    // Cleanup
110    basin
111        .delete_stream(s2_sdk::types::DeleteStreamInput::new(stream_name))
112        .await?;
113
114    println!("Streams examples completed");
115
116    // The following read session examples are for documentation snippets only.
117    // They are not executed because they would block waiting for new records.
118    if std::env::var("RUN_READ_SESSIONS").is_err() {
119        return Ok(());
120    }
121
122    // ANCHOR: read-session
123    let mut session = stream
124        .read_session(
125            ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0))),
126            ReadSessionConfig::default(),
127        )
128        .await?;
129
130    while let Some(batch) = session.next().await {
131        let batch = batch?;
132        for record in batch.records {
133            println!("[{}] {:?}", record.seq_num, record.body);
134        }
135    }
136    // ANCHOR_END: read-session
137
138    // ANCHOR: read-session-tail-offset
139    // Start reading from 10 records before the current tail
140    let mut session = stream
141        .read_session(
142            ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::TailOffset(10))),
143            ReadSessionConfig::default(),
144        )
145        .await?;
146
147    while let Some(batch) = session.next().await {
148        let batch = batch?;
149        for record in batch.records {
150            println!("[{}] {:?}", record.seq_num, record.body);
151        }
152    }
153    // ANCHOR_END: read-session-tail-offset
154
155    // ANCHOR: read-session-timestamp
156    // Start reading from a specific timestamp
157    let one_hour_ago = std::time::SystemTime::now()
158        .duration_since(std::time::UNIX_EPOCH)?
159        .as_millis() as u64
160        - 3600 * 1000;
161    let mut session = stream
162        .read_session(
163            ReadInput::new()
164                .with_start(ReadStart::new().with_from(ReadFrom::Timestamp(one_hour_ago))),
165            ReadSessionConfig::default(),
166        )
167        .await?;
168
169    while let Some(batch) = session.next().await {
170        let batch = batch?;
171        for record in batch.records {
172            println!("[{}] {:?}", record.seq_num, record.body);
173        }
174    }
175    // ANCHOR_END: read-session-timestamp
176
177    // ANCHOR: read-session-until
178    // Read records until a specific timestamp
179    let one_hour_ago = std::time::SystemTime::now()
180        .duration_since(std::time::UNIX_EPOCH)?
181        .as_millis() as u64
182        - 3600 * 1000;
183    let mut session = stream
184        .read_session(
185            ReadInput::new()
186                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
187                .with_stop(ReadStop::new().with_until(..one_hour_ago)),
188            ReadSessionConfig::default(),
189        )
190        .await?;
191
192    while let Some(batch) = session.next().await {
193        let batch = batch?;
194        for record in batch.records {
195            println!("[{}] {:?}", record.seq_num, record.body);
196        }
197    }
198    // ANCHOR_END: read-session-until
199
200    // ANCHOR: read-session-wait
201    // Read all available records, and once reaching the current tail, wait an additional 30 seconds
202    // for new ones
203    let mut session = stream
204        .read_session(
205            ReadInput::new()
206                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
207                .with_stop(ReadStop::new().with_wait(30)),
208            ReadSessionConfig::default(),
209        )
210        .await?;
211
212    while let Some(batch) = session.next().await {
213        let batch = batch?;
214        for record in batch.records {
215            println!("[{}] {:?}", record.seq_num, record.body);
216        }
217    }
218    // ANCHOR_END: read-session-wait
219
220    Ok(())
221}
pub fn with_count(self, count: usize) -> Self

Set the limit on number of records.

Examples found in repository?
examples/get_latest_record.rs (line 24)
9async fn main() -> Result<(), Box<dyn std::error::Error>> {
10    let access_token =
11        std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
12    let basin_name: BasinName = std::env::var("S2_BASIN")
13        .map_err(|_| "S2_BASIN env var not set")?
14        .parse()?;
15    let stream_name: StreamName = std::env::var("S2_STREAM")
16        .map_err(|_| "S2_STREAM env var not set")?
17        .parse()?;
18
19    let s2 = S2::new(S2Config::new(access_token))?;
20    let stream = s2.basin(basin_name).stream(stream_name);
21
22    let input = ReadInput::new()
23        .with_start(ReadStart::new().with_from(ReadFrom::TailOffset(1)))
24        .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(1)));
25    let batch = stream.read(input).await?;
26    println!("{batch:#?}");
27
28    Ok(())
29}
More examples
Hide additional examples
examples/docs_encryption.rs (line 64)
15async fn main() -> Result<(), Box<dyn std::error::Error>> {
16    let access_token = std::env::var("S2_ACCESS_TOKEN")?;
17    let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
18    let stream_name: StreamName = format!(
19        "docs-encryption-{}",
20        std::time::SystemTime::now()
21            .duration_since(std::time::UNIX_EPOCH)?
22            .as_millis()
23    )
24    .parse()?;
25
26    let client = S2::new(S2Config::new(access_token))?;
27
28    // ANCHOR: basin-cipher
29    client
30        .create_basin(
31            CreateBasinInput::new(basin_name.clone())
32                .with_config(BasinConfig::new().with_stream_cipher(EncryptionAlgorithm::Aegis256)),
33        )
34        .await?;
35
36    client
37        .reconfigure_basin(ReconfigureBasinInput::new(
38            basin_name.clone(),
39            BasinReconfiguration::new().with_stream_cipher(EncryptionAlgorithm::Aes256Gcm),
40        ))
41        .await?;
42    // ANCHOR_END: basin-cipher
43
44    let basin = client.basin(basin_name.clone());
45    basin
46        .create_stream(CreateStreamInput::new(stream_name.clone()))
47        .await?;
48
49    // ANCHOR: append-read
50    let stream = basin
51        .stream(stream_name.clone())
52        .with_encryption_key(std::env::var("S2_ENCRYPTION_KEY")?.parse()?);
53
54    stream
55        .append(AppendInput::new(AppendRecordBatch::try_from_iter([
56            AppendRecord::new("top secret")?,
57        ])?))
58        .await?;
59
60    let batch = stream
61        .read(
62            ReadInput::new()
63                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
64                .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(10))),
65        )
66        .await?;
67    // ANCHOR_END: append-read
68
69    println!("Read {} encrypted record(s)", batch.records.len());
70
71    basin
72        .delete_stream(DeleteStreamInput::new(stream_name))
73        .await?;
74
75    Ok(())
76}
examples/docs_streams.rs (line 62)
20async fn main() -> Result<(), Box<dyn std::error::Error>> {
21    let access_token = std::env::var("S2_ACCESS_TOKEN")?;
22    let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
23
24    let client = S2::new(S2Config::new(access_token))?;
25    let basin = client.basin(basin_name);
26
27    // Create a temporary stream for examples
28    let stream_name: StreamName = format!(
29        "docs-streams-{}",
30        std::time::SystemTime::now()
31            .duration_since(std::time::UNIX_EPOCH)?
32            .as_millis()
33    )
34    .parse()?;
35    basin
36        .create_stream(s2_sdk::types::CreateStreamInput::new(stream_name.clone()))
37        .await?;
38
39    // ANCHOR: simple-append
40    let stream = basin.stream(stream_name.clone());
41
42    let ack = stream
43        .append(AppendInput::new(AppendRecordBatch::try_from_iter([
44            AppendRecord::new("first event")?,
45            AppendRecord::new("second event")?,
46        ])?))
47        .await?;
48
49    // ack tells us where the records landed
50    println!(
51        "Wrote records {} through {}",
52        ack.start.seq_num,
53        ack.end.seq_num - 1
54    );
55    // ANCHOR_END: simple-append
56
57    // ANCHOR: simple-read
58    let batch = stream
59        .read(
60            ReadInput::new()
61                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
62                .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(100))),
63        )
64        .await?;
65
66    for record in batch.records {
67        println!("[{}] {:?}", record.seq_num, record.body);
68    }
69    // ANCHOR_END: simple-read
70
71    // ANCHOR: append-session
72    let session = stream.append_session(AppendSessionConfig::new());
73
74    // Submit a batch - this enqueues it and returns a ticket
75    let records = AppendRecordBatch::try_from_iter([
76        AppendRecord::new("event-1")?,
77        AppendRecord::new("event-2")?,
78    ])?;
79    let ticket = session.submit(AppendInput::new(records)).await?;
80
81    // Wait for durability
82    let ack = ticket.await?;
83    println!("Durable at seqNum {}", ack.start.seq_num);
84
85    session.close().await?;
86    // ANCHOR_END: append-session
87
88    // ANCHOR: producer
89    let producer = stream.producer(
90        ProducerConfig::new()
91            .with_batching(BatchingConfig::new().with_linger(Duration::from_millis(5))),
92    );
93
94    // Submit individual records
95    let ticket = producer.submit(AppendRecord::new("my event")?).await?;
96
97    // Get the exact sequence number
98    let ack = ticket.await?;
99    println!("Record durable at seqNum {}", ack.seq_num);
100
101    producer.close().await?;
102    // ANCHOR_END: producer
103
104    // ANCHOR: check-tail
105    let tail = stream.check_tail().await?;
106    println!("Stream has {} records", tail.seq_num);
107    // ANCHOR_END: check-tail
108
109    // Cleanup
110    basin
111        .delete_stream(s2_sdk::types::DeleteStreamInput::new(stream_name))
112        .await?;
113
114    println!("Streams examples completed");
115
116    // The following read session examples are for documentation snippets only.
117    // They are not executed because they would block waiting for new records.
118    if std::env::var("RUN_READ_SESSIONS").is_err() {
119        return Ok(());
120    }
121
122    // ANCHOR: read-session
123    let mut session = stream
124        .read_session(
125            ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0))),
126            ReadSessionConfig::default(),
127        )
128        .await?;
129
130    while let Some(batch) = session.next().await {
131        let batch = batch?;
132        for record in batch.records {
133            println!("[{}] {:?}", record.seq_num, record.body);
134        }
135    }
136    // ANCHOR_END: read-session
137
138    // ANCHOR: read-session-tail-offset
139    // Start reading from 10 records before the current tail
140    let mut session = stream
141        .read_session(
142            ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::TailOffset(10))),
143            ReadSessionConfig::default(),
144        )
145        .await?;
146
147    while let Some(batch) = session.next().await {
148        let batch = batch?;
149        for record in batch.records {
150            println!("[{}] {:?}", record.seq_num, record.body);
151        }
152    }
153    // ANCHOR_END: read-session-tail-offset
154
155    // ANCHOR: read-session-timestamp
156    // Start reading from a specific timestamp
157    let one_hour_ago = std::time::SystemTime::now()
158        .duration_since(std::time::UNIX_EPOCH)?
159        .as_millis() as u64
160        - 3600 * 1000;
161    let mut session = stream
162        .read_session(
163            ReadInput::new()
164                .with_start(ReadStart::new().with_from(ReadFrom::Timestamp(one_hour_ago))),
165            ReadSessionConfig::default(),
166        )
167        .await?;
168
169    while let Some(batch) = session.next().await {
170        let batch = batch?;
171        for record in batch.records {
172            println!("[{}] {:?}", record.seq_num, record.body);
173        }
174    }
175    // ANCHOR_END: read-session-timestamp
176
177    // ANCHOR: read-session-until
178    // Read records until a specific timestamp
179    let one_hour_ago = std::time::SystemTime::now()
180        .duration_since(std::time::UNIX_EPOCH)?
181        .as_millis() as u64
182        - 3600 * 1000;
183    let mut session = stream
184        .read_session(
185            ReadInput::new()
186                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
187                .with_stop(ReadStop::new().with_until(..one_hour_ago)),
188            ReadSessionConfig::default(),
189        )
190        .await?;
191
192    while let Some(batch) = session.next().await {
193        let batch = batch?;
194        for record in batch.records {
195            println!("[{}] {:?}", record.seq_num, record.body);
196        }
197    }
198    // ANCHOR_END: read-session-until
199
200    // ANCHOR: read-session-wait
201    // Read all available records, and once reaching the current tail, wait an additional 30 seconds
202    // for new ones
203    let mut session = stream
204        .read_session(
205            ReadInput::new()
206                .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
207                .with_stop(ReadStop::new().with_wait(30)),
208            ReadSessionConfig::default(),
209        )
210        .await?;
211
212    while let Some(batch) = session.next().await {
213        let batch = batch?;
214        for record in batch.records {
215            println!("[{}] {:?}", record.seq_num, record.body);
216        }
217    }
218    // ANCHOR_END: read-session-wait
219
220    Ok(())
221}
pub fn with_bytes(self, bytes: usize) -> Self

Set the limit on total metered bytes of records.