#[non_exhaustive]pub struct S2Config { /* private fields */ }Expand description
Configuration for S2.
Implementations
Sourceimpl S2Config
impl S2Config
Sourcepub fn new(access_token: impl Into<String>) -> Self
pub fn new(access_token: impl Into<String>) -> Self
Create a new S2Config with the given access token and default settings.
Examples found in repository?
5fn main() -> Result<(), Box<dyn std::error::Error>> {
6 // ANCHOR: create-client
7 use s2_sdk::{S2, types::S2Config};
8
9 let client = S2::new(S2Config::new(std::env::var("S2_ACCESS_TOKEN")?))?;
10
11 let basin = client.basin("my-basin".parse()?);
12 let stream = basin.stream("my-stream".parse()?);
13 // ANCHOR_END: create-client
14
15 println!("Created client for stream: {:?}", stream);
16 Ok(())
17}More examples
8async fn main() -> Result<(), Box<dyn std::error::Error>> {
9 let access_token =
10 std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
11
12 let config = S2Config::new(access_token);
13 let s2 = S2::new(config)?;
14
15 let input = ListAllBasinsInput::new();
16
17 let basins: Vec<_> = s2.list_all_basins(input).take(10).try_collect().await?;
18
19 println!("{basins:#?}");
20
21 Ok(())
22}7async fn main() -> Result<(), Box<dyn std::error::Error>> {
8 let access_token =
9 std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
10 let basin_name: BasinName = std::env::var("S2_BASIN")
11 .map_err(|_| "S2_BASIN env var not set")?
12 .parse()?;
13
14 let config = S2Config::new(access_token);
15 let s2 = S2::new(config)?;
16
17 let input = DeleteBasinInput::new(basin_name).with_ignore_not_found(true);
18 s2.delete_basin(input).await?;
19 println!("Deletion requested");
20
21 Ok(())
22}7async fn main() -> Result<(), Box<dyn std::error::Error>> {
8 let access_token =
9 std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
10 let basin_name: BasinName = std::env::var("S2_BASIN")
11 .map_err(|_| "S2_BASIN env var not set")?
12 .parse()?;
13
14 let s2 = S2::new(S2Config::new(access_token))?;
15 let basin = s2.basin(basin_name);
16
17 let input = ListStreamsInput::new().with_prefix("my-".parse()?);
18 let page = basin.list_streams(input).await?;
19 println!("{page:#?}");
20
21 Ok(())
22}7async fn main() -> Result<(), Box<dyn std::error::Error>> {
8 let access_token =
9 std::env::var("S2_ACCESS_TOKEN").map_err(|_| "S2_ACCESS_TOKEN env var not set")?;
10 let basin_name: BasinName = std::env::var("S2_BASIN")
11 .map_err(|_| "S2_BASIN env var not set")?
12 .parse()?;
13 let stream_name: StreamName = std::env::var("S2_STREAM")
14 .map_err(|_| "S2_STREAM env var not set")?
15 .parse()?;
16
17 let s2 = S2::new(S2Config::new(access_token))?;
18 let basin = s2.basin(basin_name);
19
20 let input = DeleteStreamInput::new(stream_name);
21 basin.delete_stream(input).await?;
22 println!("Deletion requested");
23
24 Ok(())
25}9async fn main() -> Result<(), Box<dyn std::error::Error>> {
10 let access_token = std::env::var("S2_ACCESS_TOKEN")?;
11 let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
12 let stream_name: StreamName = std::env::var("S2_STREAM")?.parse()?;
13
14 let s2 = S2::new(S2Config::new(access_token))?;
15 let stream = s2.basin(basin_name).stream(stream_name);
16
17 let input = ReadInput::new();
18 let mut batches = stream.read_session(input).await?;
19 loop {
20 select! {
21 batch = batches.next() => {
22 let Some(batch) = batch else { break };
23 let batch = batch?;
24 println!("{batch:?}");
25 }
26 _ = tokio::signal::ctrl_c() => break,
27 }
28 }
29
30 Ok(())
31}- examples/reconfigure_stream.rs
- examples/create_basin.rs
- examples/get_latest_record.rs
- examples/reconfigure_basin.rs
- examples/explicit_trim.rs
- examples/issue_access_token.rs
- examples/create_stream.rs
- examples/producer.rs
- examples/docs_metrics.rs
- examples/docs_configuration.rs
- examples/docs_account_and_basins.rs
- examples/docs_streams.rs
Sourcepub fn with_endpoints(self, endpoints: S2Endpoints) -> Self
pub fn with_endpoints(self, endpoints: S2Endpoints) -> Self
Set the S2 endpoints to connect to.
Examples found in repository?
12fn main() -> Result<(), Box<dyn std::error::Error>> {
13 // Example: Custom endpoints (e.g., for s2-lite local dev)
14 {
15 // ANCHOR: custom-endpoints
16 let client = S2::new(
17 S2Config::new("local-token").with_endpoints(S2Endpoints::new(
18 AccountEndpoint::new("http://localhost:8080")?,
19 BasinEndpoint::new("http://localhost:8080")?,
20 )?),
21 )?;
22 // ANCHOR_END: custom-endpoints
23 println!("Created client with custom endpoints: {:?}", client);
24 }
25
26 // Example: Custom retry configuration
27 {
28 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
29 // ANCHOR: retry-config
30 let client = S2::new(
31 S2Config::new(access_token).with_retry(
32 RetryConfig::new()
33 .with_max_attempts(NonZeroU32::new(5).unwrap())
34 .with_min_base_delay(Duration::from_millis(100))
35 .with_max_base_delay(Duration::from_secs(2)),
36 ),
37 )?;
38 // ANCHOR_END: retry-config
39 println!("Created client with retry config: {:?}", client);
40 }
41
42 // Example: Custom timeout configuration
43 {
44 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
45 // ANCHOR: timeout-config
46 let client = S2::new(
47 S2Config::new(access_token)
48 .with_connection_timeout(Duration::from_secs(5))
49 .with_request_timeout(Duration::from_secs(10)),
50 )?;
51 // ANCHOR_END: timeout-config
52 println!("Created client with timeout config: {:?}", client);
53 }
54
55 Ok(())
56}Sourcepub fn with_connection_timeout(self, connection_timeout: Duration) -> Self
pub fn with_connection_timeout(self, connection_timeout: Duration) -> Self
Set the timeout for establishing a connection to the server.
Defaults to 3s.
Examples found in repository?
12fn main() -> Result<(), Box<dyn std::error::Error>> {
13 // Example: Custom endpoints (e.g., for s2-lite local dev)
14 {
15 // ANCHOR: custom-endpoints
16 let client = S2::new(
17 S2Config::new("local-token").with_endpoints(S2Endpoints::new(
18 AccountEndpoint::new("http://localhost:8080")?,
19 BasinEndpoint::new("http://localhost:8080")?,
20 )?),
21 )?;
22 // ANCHOR_END: custom-endpoints
23 println!("Created client with custom endpoints: {:?}", client);
24 }
25
26 // Example: Custom retry configuration
27 {
28 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
29 // ANCHOR: retry-config
30 let client = S2::new(
31 S2Config::new(access_token).with_retry(
32 RetryConfig::new()
33 .with_max_attempts(NonZeroU32::new(5).unwrap())
34 .with_min_base_delay(Duration::from_millis(100))
35 .with_max_base_delay(Duration::from_secs(2)),
36 ),
37 )?;
38 // ANCHOR_END: retry-config
39 println!("Created client with retry config: {:?}", client);
40 }
41
42 // Example: Custom timeout configuration
43 {
44 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
45 // ANCHOR: timeout-config
46 let client = S2::new(
47 S2Config::new(access_token)
48 .with_connection_timeout(Duration::from_secs(5))
49 .with_request_timeout(Duration::from_secs(10)),
50 )?;
51 // ANCHOR_END: timeout-config
52 println!("Created client with timeout config: {:?}", client);
53 }
54
55 Ok(())
56}Sourcepub fn with_request_timeout(self, request_timeout: Duration) -> Self
pub fn with_request_timeout(self, request_timeout: Duration) -> Self
Set the timeout for requests.
Defaults to 5s.
Examples found in repository?
12fn main() -> Result<(), Box<dyn std::error::Error>> {
13 // Example: Custom endpoints (e.g., for s2-lite local dev)
14 {
15 // ANCHOR: custom-endpoints
16 let client = S2::new(
17 S2Config::new("local-token").with_endpoints(S2Endpoints::new(
18 AccountEndpoint::new("http://localhost:8080")?,
19 BasinEndpoint::new("http://localhost:8080")?,
20 )?),
21 )?;
22 // ANCHOR_END: custom-endpoints
23 println!("Created client with custom endpoints: {:?}", client);
24 }
25
26 // Example: Custom retry configuration
27 {
28 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
29 // ANCHOR: retry-config
30 let client = S2::new(
31 S2Config::new(access_token).with_retry(
32 RetryConfig::new()
33 .with_max_attempts(NonZeroU32::new(5).unwrap())
34 .with_min_base_delay(Duration::from_millis(100))
35 .with_max_base_delay(Duration::from_secs(2)),
36 ),
37 )?;
38 // ANCHOR_END: retry-config
39 println!("Created client with retry config: {:?}", client);
40 }
41
42 // Example: Custom timeout configuration
43 {
44 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
45 // ANCHOR: timeout-config
46 let client = S2::new(
47 S2Config::new(access_token)
48 .with_connection_timeout(Duration::from_secs(5))
49 .with_request_timeout(Duration::from_secs(10)),
50 )?;
51 // ANCHOR_END: timeout-config
52 println!("Created client with timeout config: {:?}", client);
53 }
54
55 Ok(())
56}Sourcepub fn with_retry(self, retry: RetryConfig) -> Self
pub fn with_retry(self, retry: RetryConfig) -> Self
Set the retry configuration for requests.
See RetryConfig for defaults.
Examples found in repository?
12fn main() -> Result<(), Box<dyn std::error::Error>> {
13 // Example: Custom endpoints (e.g., for s2-lite local dev)
14 {
15 // ANCHOR: custom-endpoints
16 let client = S2::new(
17 S2Config::new("local-token").with_endpoints(S2Endpoints::new(
18 AccountEndpoint::new("http://localhost:8080")?,
19 BasinEndpoint::new("http://localhost:8080")?,
20 )?),
21 )?;
22 // ANCHOR_END: custom-endpoints
23 println!("Created client with custom endpoints: {:?}", client);
24 }
25
26 // Example: Custom retry configuration
27 {
28 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
29 // ANCHOR: retry-config
30 let client = S2::new(
31 S2Config::new(access_token).with_retry(
32 RetryConfig::new()
33 .with_max_attempts(NonZeroU32::new(5).unwrap())
34 .with_min_base_delay(Duration::from_millis(100))
35 .with_max_base_delay(Duration::from_secs(2)),
36 ),
37 )?;
38 // ANCHOR_END: retry-config
39 println!("Created client with retry config: {:?}", client);
40 }
41
42 // Example: Custom timeout configuration
43 {
44 let access_token = std::env::var("S2_ACCESS_TOKEN").unwrap_or_else(|_| "demo".into());
45 // ANCHOR: timeout-config
46 let client = S2::new(
47 S2Config::new(access_token)
48 .with_connection_timeout(Duration::from_secs(5))
49 .with_request_timeout(Duration::from_secs(10)),
50 )?;
51 // ANCHOR_END: timeout-config
52 println!("Created client with timeout config: {:?}", client);
53 }
54
55 Ok(())
56}Sourcepub fn with_compression(self, compression: Compression) -> Self
pub fn with_compression(self, compression: Compression) -> Self
Set the compression algorithm for requests and responses.
Defaults to no compression.
Sourcepub fn with_insecure_skip_cert_verification(self, skip: bool) -> Self
pub fn with_insecure_skip_cert_verification(self, skip: bool) -> Self
Skip TLS certificate verification (insecure).
This is useful for connecting to endpoints with self-signed certificates
or certificates that don’t match the hostname (similar to curl -k).
Warning
This disables certificate verification and should only be used for testing or development purposes. Never use this in production.
Defaults to false.
Sourcepub fn with_rustls_crypto_provider(
self,
provider: impl Into<Arc<CryptoProvider>>,
) -> Self
pub fn with_rustls_crypto_provider( self, provider: impl Into<Arc<CryptoProvider>>, ) -> Self
Use a specific rustls crypto provider for SDK TLS connections.
With default features enabled, the SDK uses the aws-lc-rs provider.
With default features disabled, the SDK uses rustls’s process-global
provider if one has been installed, or returns an error otherwise.
Use this when your application needs a specific rustls provider, such as
ring or a custom rustls::crypto::CryptoProvider. The corresponding
rustls provider feature must be enabled in the dependency graph.
Sourcepub fn with_rustls_aws_lc_rs_crypto_provider(self) -> Self
pub fn with_rustls_aws_lc_rs_crypto_provider(self) -> Self
Use rustls’s aws-lc-rs crypto provider.
Requires the rustls-aws-lc-rs crate feature.
Sourcepub fn with_rustls_ring_crypto_provider(self) -> Self
pub fn with_rustls_ring_crypto_provider(self) -> Self
Use rustls’s ring crypto provider.
Requires the rustls-ring crate feature.