pub type StreamName = StreamNameStr<NameProps>;Expand description
Stream name.
Note: It must be unique to the basin and between 1 and 512 bytes in length, and must not contain NUL bytes.
Aliased Type
pub struct StreamName(/* private fields */);pub type StreamName = StreamNameStr<NameProps>;Stream name.
Note: It must be unique to the basin and between 1 and 512 bytes in length, and must not contain NUL bytes.
pub struct StreamName(/* private fields */);