pub struct BatchingConfig { /* private fields */ }Expand description
Configuration for batching AppendRecords.
Implementations
Sourceimpl BatchingConfig
impl BatchingConfig
Sourcepub fn new() -> Self
pub fn new() -> Self
Create a new BatchingConfig with default settings.
Examples found in repository?
examples/docs_streams.rs (line 91)
20async fn main() -> Result<(), Box<dyn std::error::Error>> {
21 let access_token = std::env::var("S2_ACCESS_TOKEN")?;
22 let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
23
24 let client = S2::new(S2Config::new(access_token))?;
25 let basin = client.basin(basin_name);
26
27 // Create a temporary stream for examples
28 let stream_name: StreamName = format!(
29 "docs-streams-{}",
30 std::time::SystemTime::now()
31 .duration_since(std::time::UNIX_EPOCH)?
32 .as_millis()
33 )
34 .parse()?;
35 basin
36 .create_stream(s2_sdk::types::CreateStreamInput::new(stream_name.clone()))
37 .await?;
38
39 // ANCHOR: simple-append
40 let stream = basin.stream(stream_name.clone());
41
42 let ack = stream
43 .append(AppendInput::new(AppendRecordBatch::try_from_iter([
44 AppendRecord::new("first event")?,
45 AppendRecord::new("second event")?,
46 ])?))
47 .await?;
48
49 // ack tells us where the records landed
50 println!(
51 "Wrote records {} through {}",
52 ack.start.seq_num,
53 ack.end.seq_num - 1
54 );
55 // ANCHOR_END: simple-append
56
57 // ANCHOR: simple-read
58 let batch = stream
59 .read(
60 ReadInput::new()
61 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
62 .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(100))),
63 )
64 .await?;
65
66 for record in batch.records {
67 println!("[{}] {:?}", record.seq_num, record.body);
68 }
69 // ANCHOR_END: simple-read
70
71 // ANCHOR: append-session
72 let session = stream.append_session(AppendSessionConfig::new());
73
74 // Submit a batch - this enqueues it and returns a ticket
75 let records = AppendRecordBatch::try_from_iter([
76 AppendRecord::new("event-1")?,
77 AppendRecord::new("event-2")?,
78 ])?;
79 let ticket = session.submit(AppendInput::new(records)).await?;
80
81 // Wait for durability
82 let ack = ticket.await?;
83 println!("Durable at seqNum {}", ack.start.seq_num);
84
85 session.close().await?;
86 // ANCHOR_END: append-session
87
88 // ANCHOR: producer
89 let producer = stream.producer(
90 ProducerConfig::new()
91 .with_batching(BatchingConfig::new().with_linger(Duration::from_millis(5))),
92 );
93
94 // Submit individual records
95 let ticket = producer.submit(AppendRecord::new("my event")?).await?;
96
97 // Force the partial batch and wait for durability without closing the producer
98 producer.flush().await?;
99
100 // Get the exact sequence number
101 let ack = ticket.await?;
102 println!("Record durable at seqNum {}", ack.seq_num);
103
104 producer.close().await?;
105 // ANCHOR_END: producer
106
107 // ANCHOR: check-tail
108 let tail = stream.check_tail().await?;
109 println!("Stream has {} records", tail.seq_num);
110 // ANCHOR_END: check-tail
111
112 // Cleanup
113 basin
114 .delete_stream(s2_sdk::types::DeleteStreamInput::new(stream_name))
115 .await?;
116
117 println!("Streams examples completed");
118
119 // The following read session examples are for documentation snippets only.
120 // They are not executed because they would block waiting for new records.
121 if std::env::var("RUN_READ_SESSIONS").is_err() {
122 return Ok(());
123 }
124
125 // ANCHOR: read-session
126 let mut session = stream
127 .read_session(
128 ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0))),
129 ReadSessionConfig::default(),
130 )
131 .await?;
132
133 while let Some(batch) = session.next().await {
134 let batch = batch?;
135 for record in batch.records {
136 println!("[{}] {:?}", record.seq_num, record.body);
137 }
138 }
139 // ANCHOR_END: read-session
140
141 // ANCHOR: read-session-tail-offset
142 // Start reading from 10 records before the current tail
143 let mut session = stream
144 .read_session(
145 ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::TailOffset(10))),
146 ReadSessionConfig::default(),
147 )
148 .await?;
149
150 while let Some(batch) = session.next().await {
151 let batch = batch?;
152 for record in batch.records {
153 println!("[{}] {:?}", record.seq_num, record.body);
154 }
155 }
156 // ANCHOR_END: read-session-tail-offset
157
158 // ANCHOR: read-session-timestamp
159 // Start reading from a specific timestamp
160 let one_hour_ago = std::time::SystemTime::now()
161 .duration_since(std::time::UNIX_EPOCH)?
162 .as_millis() as u64
163 - 3600 * 1000;
164 let mut session = stream
165 .read_session(
166 ReadInput::new()
167 .with_start(ReadStart::new().with_from(ReadFrom::Timestamp(one_hour_ago))),
168 ReadSessionConfig::default(),
169 )
170 .await?;
171
172 while let Some(batch) = session.next().await {
173 let batch = batch?;
174 for record in batch.records {
175 println!("[{}] {:?}", record.seq_num, record.body);
176 }
177 }
178 // ANCHOR_END: read-session-timestamp
179
180 // ANCHOR: read-session-until
181 // Read records until a specific timestamp
182 let one_hour_ago = std::time::SystemTime::now()
183 .duration_since(std::time::UNIX_EPOCH)?
184 .as_millis() as u64
185 - 3600 * 1000;
186 let mut session = stream
187 .read_session(
188 ReadInput::new()
189 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
190 .with_stop(ReadStop::new().with_until(..one_hour_ago)),
191 ReadSessionConfig::default(),
192 )
193 .await?;
194
195 while let Some(batch) = session.next().await {
196 let batch = batch?;
197 for record in batch.records {
198 println!("[{}] {:?}", record.seq_num, record.body);
199 }
200 }
201 // ANCHOR_END: read-session-until
202
203 // ANCHOR: read-session-wait
204 // Read all available records, and once reaching the current tail, wait an additional 30 seconds
205 // for new ones
206 let mut session = stream
207 .read_session(
208 ReadInput::new()
209 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
210 .with_stop(ReadStop::new().with_wait(30)),
211 ReadSessionConfig::default(),
212 )
213 .await?;
214
215 while let Some(batch) = session.next().await {
216 let batch = batch?;
217 for record in batch.records {
218 println!("[{}] {:?}", record.seq_num, record.body);
219 }
220 }
221 // ANCHOR_END: read-session-wait
222
223 Ok(())
224}Sourcepub fn with_linger(self, linger: Duration) -> Self
pub fn with_linger(self, linger: Duration) -> Self
Set the duration for how long to wait for more records before flushing a batch.
Defaults to 5ms.
Examples found in repository?
examples/docs_streams.rs (line 91)
20async fn main() -> Result<(), Box<dyn std::error::Error>> {
21 let access_token = std::env::var("S2_ACCESS_TOKEN")?;
22 let basin_name: BasinName = std::env::var("S2_BASIN")?.parse()?;
23
24 let client = S2::new(S2Config::new(access_token))?;
25 let basin = client.basin(basin_name);
26
27 // Create a temporary stream for examples
28 let stream_name: StreamName = format!(
29 "docs-streams-{}",
30 std::time::SystemTime::now()
31 .duration_since(std::time::UNIX_EPOCH)?
32 .as_millis()
33 )
34 .parse()?;
35 basin
36 .create_stream(s2_sdk::types::CreateStreamInput::new(stream_name.clone()))
37 .await?;
38
39 // ANCHOR: simple-append
40 let stream = basin.stream(stream_name.clone());
41
42 let ack = stream
43 .append(AppendInput::new(AppendRecordBatch::try_from_iter([
44 AppendRecord::new("first event")?,
45 AppendRecord::new("second event")?,
46 ])?))
47 .await?;
48
49 // ack tells us where the records landed
50 println!(
51 "Wrote records {} through {}",
52 ack.start.seq_num,
53 ack.end.seq_num - 1
54 );
55 // ANCHOR_END: simple-append
56
57 // ANCHOR: simple-read
58 let batch = stream
59 .read(
60 ReadInput::new()
61 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
62 .with_stop(ReadStop::new().with_limits(ReadLimits::new().with_count(100))),
63 )
64 .await?;
65
66 for record in batch.records {
67 println!("[{}] {:?}", record.seq_num, record.body);
68 }
69 // ANCHOR_END: simple-read
70
71 // ANCHOR: append-session
72 let session = stream.append_session(AppendSessionConfig::new());
73
74 // Submit a batch - this enqueues it and returns a ticket
75 let records = AppendRecordBatch::try_from_iter([
76 AppendRecord::new("event-1")?,
77 AppendRecord::new("event-2")?,
78 ])?;
79 let ticket = session.submit(AppendInput::new(records)).await?;
80
81 // Wait for durability
82 let ack = ticket.await?;
83 println!("Durable at seqNum {}", ack.start.seq_num);
84
85 session.close().await?;
86 // ANCHOR_END: append-session
87
88 // ANCHOR: producer
89 let producer = stream.producer(
90 ProducerConfig::new()
91 .with_batching(BatchingConfig::new().with_linger(Duration::from_millis(5))),
92 );
93
94 // Submit individual records
95 let ticket = producer.submit(AppendRecord::new("my event")?).await?;
96
97 // Force the partial batch and wait for durability without closing the producer
98 producer.flush().await?;
99
100 // Get the exact sequence number
101 let ack = ticket.await?;
102 println!("Record durable at seqNum {}", ack.seq_num);
103
104 producer.close().await?;
105 // ANCHOR_END: producer
106
107 // ANCHOR: check-tail
108 let tail = stream.check_tail().await?;
109 println!("Stream has {} records", tail.seq_num);
110 // ANCHOR_END: check-tail
111
112 // Cleanup
113 basin
114 .delete_stream(s2_sdk::types::DeleteStreamInput::new(stream_name))
115 .await?;
116
117 println!("Streams examples completed");
118
119 // The following read session examples are for documentation snippets only.
120 // They are not executed because they would block waiting for new records.
121 if std::env::var("RUN_READ_SESSIONS").is_err() {
122 return Ok(());
123 }
124
125 // ANCHOR: read-session
126 let mut session = stream
127 .read_session(
128 ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0))),
129 ReadSessionConfig::default(),
130 )
131 .await?;
132
133 while let Some(batch) = session.next().await {
134 let batch = batch?;
135 for record in batch.records {
136 println!("[{}] {:?}", record.seq_num, record.body);
137 }
138 }
139 // ANCHOR_END: read-session
140
141 // ANCHOR: read-session-tail-offset
142 // Start reading from 10 records before the current tail
143 let mut session = stream
144 .read_session(
145 ReadInput::new().with_start(ReadStart::new().with_from(ReadFrom::TailOffset(10))),
146 ReadSessionConfig::default(),
147 )
148 .await?;
149
150 while let Some(batch) = session.next().await {
151 let batch = batch?;
152 for record in batch.records {
153 println!("[{}] {:?}", record.seq_num, record.body);
154 }
155 }
156 // ANCHOR_END: read-session-tail-offset
157
158 // ANCHOR: read-session-timestamp
159 // Start reading from a specific timestamp
160 let one_hour_ago = std::time::SystemTime::now()
161 .duration_since(std::time::UNIX_EPOCH)?
162 .as_millis() as u64
163 - 3600 * 1000;
164 let mut session = stream
165 .read_session(
166 ReadInput::new()
167 .with_start(ReadStart::new().with_from(ReadFrom::Timestamp(one_hour_ago))),
168 ReadSessionConfig::default(),
169 )
170 .await?;
171
172 while let Some(batch) = session.next().await {
173 let batch = batch?;
174 for record in batch.records {
175 println!("[{}] {:?}", record.seq_num, record.body);
176 }
177 }
178 // ANCHOR_END: read-session-timestamp
179
180 // ANCHOR: read-session-until
181 // Read records until a specific timestamp
182 let one_hour_ago = std::time::SystemTime::now()
183 .duration_since(std::time::UNIX_EPOCH)?
184 .as_millis() as u64
185 - 3600 * 1000;
186 let mut session = stream
187 .read_session(
188 ReadInput::new()
189 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
190 .with_stop(ReadStop::new().with_until(..one_hour_ago)),
191 ReadSessionConfig::default(),
192 )
193 .await?;
194
195 while let Some(batch) = session.next().await {
196 let batch = batch?;
197 for record in batch.records {
198 println!("[{}] {:?}", record.seq_num, record.body);
199 }
200 }
201 // ANCHOR_END: read-session-until
202
203 // ANCHOR: read-session-wait
204 // Read all available records, and once reaching the current tail, wait an additional 30 seconds
205 // for new ones
206 let mut session = stream
207 .read_session(
208 ReadInput::new()
209 .with_start(ReadStart::new().with_from(ReadFrom::SeqNum(0)))
210 .with_stop(ReadStop::new().with_wait(30)),
211 ReadSessionConfig::default(),
212 )
213 .await?;
214
215 while let Some(batch) = session.next().await {
216 let batch = batch?;
217 for record in batch.records {
218 println!("[{}] {:?}", record.seq_num, record.body);
219 }
220 }
221 // ANCHOR_END: read-session-wait
222
223 Ok(())
224}Sourcepub fn with_limits(self, limits: BatchLimits) -> Self
pub fn with_limits(self, limits: BatchLimits) -> Self
Set the batch limits.